import 'package:args/args.dart';
import 'package:args/command_runner.dart';
import 'package:cli_completion/cli_completion.dart';
import 'package:globe_cli/src/utils/api.dart';
import 'package:mason_logger/mason_logger.dart';
import 'package:pub_updater/pub_updater.dart';
import 'package:globe_cli/src/commands/commands.dart';
import 'package:globe_cli/src/version.dart';

const executableName = 'globe';
const packageName = 'globe_cli';
const description = 'The Dart Globe CLI tool';

/// {@template globe_cli}
/// A [CommandRunner] for the CLI.
///
/// ```
/// $ deploy --version
/// ```
/// {@endtemplate}
class GlobeCliCommandRunner extends CompletionCommandRunner<int> {
  /// {@macro globe_cli}
  GlobeCliCommandRunner({
    PubUpdater? pubUpdater,
    Logger? logger,
    GlobeApi? api,
  })  : _logger = logger ?? Logger(),
        _api = api ?? GlobeApi(),
        _pubUpdater = pubUpdater ?? PubUpdater(),
        super(executableName, description) {
    argParser
      ..addFlag(
        'version',
        abbr: 'v',
        negatable: false,
        help: 'Print the current version of the CLI.',
      )
      ..addFlag(
        'verbose',
        help: 'Enables verbose logging.',
      )
      ..addFlag(
        'local',
        help: 'Switches the CLI to use a locally running API.',
      );

    addCommand(LoginCommand(logger: _logger, api: _api));
    addCommand(LogoutCommand(logger: _logger, api: _api));
    addCommand(DeployCommand(logger: _logger, api: _api));
  }

  @override
  void printUsage() => _logger.info(usage);

  final GlobeApi _api;
  final Logger _logger;
  final PubUpdater _pubUpdater;

  @override
  Future<int> run(Iterable<String> args) async {
    _logger.info('Globe CLI vTODO');

    try {
      final topLevelResults = parse(args);
      if (topLevelResults['verbose'] == true) {
        _logger.level = Level.verbose;
      }
      if (topLevelResults['local'] == true) {
        _logger.warn(
            'You are using a local API. Remove the --local flag to use the production API.');
        _api.endpoint = 'http://localhost:3000';
      }
      return await runCommand(topLevelResults) ?? ExitCode.success.code;
    } on FormatException catch (e, stackTrace) {
      // On format errors, show the commands error message, root usage and
      // exit with an error code
      _logger
        ..err(e.message)
        ..err('$stackTrace')
        ..info('')
        ..info(usage);
      return ExitCode.usage.code;
    } on UsageException catch (e) {
      // On usage errors, show the commands usage message and
      // exit with an error code
      _logger
        ..err(e.message)
        ..info('')
        ..info(e.usage);
      return ExitCode.usage.code;
    }
  }

  @override
  Future<int?> runCommand(ArgResults topLevelResults) async {
    // Fast track completion command
    if (topLevelResults.command?.name == 'completion') {
      await super.runCommand(topLevelResults);
      return ExitCode.success.code;
    }

    // Run the command or show version
    final int? exitCode;
    if (topLevelResults['version'] == true) {
      _logger.info(packageVersion);
      exitCode = ExitCode.success.code;
    } else {
      exitCode = await super.runCommand(topLevelResults);
    }

    // Check for updates
    // if (topLevelResults.command?.name != UpdateCommand.commandName) {
    //   await _checkForUpdates();
    // }

    return exitCode;
  }

  // Checks if the current version (set by the build runner on the
  // version.dart file) is the most recent one. If not, show a prompt to the
  // user.
  Future<void> _checkForUpdates() async {
    try {
      final latestVersion = await _pubUpdater.getLatestVersion(packageName);
      final isUpToDate = packageVersion == latestVersion;
      if (!isUpToDate) {
        _logger
          ..info('')
          ..info(
            '''
${lightYellow.wrap('Update available!')} ${lightCyan.wrap(packageVersion)} \u2192 ${lightCyan.wrap(latestVersion)}
Run ${lightCyan.wrap('$executableName update')} to update''',
          );
      }
    } catch (_) {}
  }
}
