import 'dart:io';

import 'package:globe_cli/src/command.dart';
import 'package:globe_cli/src/utils/archiver.dart';
import 'package:mason_logger/mason_logger.dart';

import '../utils/api.dart';
import '../utils/prompts.dart';
import '../utils/scope.dart';

/// {@template deploy_command}
///
/// `dart_deploy logout`
/// Logout of the current user.
/// {@endtemplate}
class DeployCommand extends BaseGlobeCommand {
  /// {@macro deploy_command}
  DeployCommand({
    super.api,
    super.auth,
    super.logger,
    super.runProcess,
    super.startProcess,
  }) : _scope = Scope();

  final Scope _scope;

  @override
  String get description => 'Deploy the current project.';

  @override
  String get name => 'deploy';

  @override
  Future<int> run() async {
    requireAuth();

    if (_scope.current == null) {
      if (logger.confirm(
              'Set up and deploy ${cyan.wrap('"${Directory.current.path}"')}?',
              defaultValue: true) ==
          false) {
        logger.write('Cancelled deployment.');
        return ExitCode.success.code;
      }

      try {
        final organization = await selectOrganization(api: api, logger: logger);
        final project =
            await selectProject(organization, api: api, logger: logger);

        _scope.setScope(orgId: organization.id, projectId: project.id);
        logger.success('Updated scope.');
      } on ApiException catch (e) {
        logger.err(e.message);
        return ExitCode.software.code;
      } catch (e) {
        logger.err(e.toString());
        return ExitCode.software.code;
      }
    }

    // Get the current project metadata.
    final metadata = _scope.current!;

    late final Organization organization;
    late final Project project;

    try {
      final organizations = await api.getOrganizations();
      organization = organizations.firstWhere(
        (org) => org.id == metadata.orgId,
        orElse: () => throw Exception(
            'You do not have permission to access this organization.'),
      );
      final projects = await api.getProjects(org: organization.id);
      project = projects.firstWhere(
        (project) => project.id == metadata.projectId,
        orElse: () => throw Exception(
            'You do not have permission to access this project.'),
      );
    } on ApiException catch (e) {
      logger.err(e.message);
      return ExitCode.software.code;
    } on Exception catch (e) {
      logger.err(e.toString());
      return ExitCode.software.code;
    }

    final deployProgess = logger.progress('Deploying project...');

    // TODO: handle args for deploy command (e.g. --release, --debug, --pub-cache etc.)
    final archiver = Archiver(directory: Directory.current);

    try {
      // Archive the current directory.
      final archive = await archiver.archive();

      // Deploy the project via the API.
      final buildId = await api.deploy(
        orgId: organization.id,
        projectId: project.id,
        regionCode: 'lhr',
        archive: archive,
      );

      deployProgess.complete();
      logger.success('Deployed project successfully!');
      logger.success(
          'Visit: ${api.endpoint}/${organization.slug}/${project.slug}/builds/$buildId');
      return ExitCode.success.code;
    } on ApiException catch (e) {
      deployProgess.fail();
      logger.err('Failed to deploy project: ${e.message}');
      return ExitCode.software.code;
    } catch (e, s) {
      deployProgess.fail();
      logger
        ..err('Failed to deploy project: $e')
        ..err(s.toString());
      return ExitCode.software.code;
    }
  }
}
