import 'dart:convert';

import 'package:globe_cli/src/utils/auth.dart';
import 'package:http/http.dart' as http;

class ApiException implements Exception {
  int statusCode;
  String message;
  ApiException(this.statusCode, this.message);
}

class GlobeApi {
  String endpoint;
  final GlobeAuth _auth;

  GlobeApi({
    GlobeAuth? auth,
    String? endpoint,
  })  : _auth = auth ?? GlobeAuth(),
        endpoint = endpoint ?? 'https://globe.dev';

  /// Creates a [Uri] from the given [path], using the [_endpoint] as the base.
  Uri _buildUri(String path) {
    return Uri.parse('$endpoint/api$path');
  }

  /// Wraps a request with error handling and JSON parsing.
  Future<T> _onRequest<T extends Object>(Future<http.Response> request) async {
    final response = await request;
    String body = response.body;
    Map<String, dynamic> json;

    try {
      json = jsonDecode(body) as Map<String, dynamic>;
    } catch (e) {
      throw Exception('Invalid JSON response from API: $body');
    }

    if (response.statusCode == 200) {
      return json['data'] as T;
    }

    throw ApiException(response.statusCode, json['message'] as String);
  }

  void requireAuth() {
    if (_auth.currentSession == null) {
      throw Exception('The user is not authenticated.');
    }
  }

  Map<String, String> get headers {
    return {
      'X-Globe-Platform': 'globe_cli',
      if (_auth.currentSession != null)
        'Authorization': 'Bearer ${_auth.currentSession!.jwt}',
    };
  }

  /// Gets all of the organizations that the current user is a member of.
  Future<List<Organization>> getOrganizations() async {
    requireAuth();
    final response = await _onRequest<List>(
        http.get(_buildUri('/user/orgs'), headers: headers));

    return response.map((org) {
      return Organization.fromJson(org);
    }).toList();
  }

  /// Gets all of the projects that the current user is a member of.
  Future<List<Project>> getProjects({
    required String org,
  }) async {
    requireAuth();
    final response = await _onRequest<List>(
        http.get(_buildUri('/org/$org/projects'), headers: headers));

    return response.map((project) {
      return Project.fromJson(project);
    }).toList();
  }

  /// Creates a new project and returns it.
  Future<Project> createProject({
    required String orgId,
    required String name,
  }) async {
    final request =
        http.MultipartRequest('POST', _buildUri('/org/$orgId/projects'));
    request.headers.addAll(headers);
    request.fields['name'] = name;

    final response =
        await _onRequest<Map>(request.send().then(http.Response.fromStream));

    return Project.fromJson(response);
  }

  /// Deploy the current project to the given project.
  Future<String> deploy({
    required String orgId,
    required String projectId,
    required String regionCode,
    required List<int> archive,
  }) async {
    requireAuth();

    final request = http.MultipartRequest(
        'POST', _buildUri('/org/$orgId/projects/$projectId/deploy'));

    request.headers.addAll(headers);
    request.fields['regionCode'] = regionCode;

    // TODO: add max size check

    request.files.add(
      http.MultipartFile.fromBytes(
        'projectArchive',
        archive,
        filename: 'archive.zip',
      ),
    );

    final response =
        await _onRequest<Map>(request.send().then(http.Response.fromStream));

    return response['id'];
  }
}

class Organization {
  final String id;
  final String name;
  final String slug;
  final OrganizationType type;
  final Role role;
  final DateTime createdAt;
  final DateTime updatedAt;

  Organization._({
    required this.id,
    required this.name,
    required this.slug,
    required this.type,
    required this.role,
    required this.createdAt,
    required this.updatedAt,
  });

  factory Organization.fromJson(Map<dynamic, dynamic> json) {
    return Organization._(
      id: json['id'],
      name: json['name'],
      slug: json['slug'],
      type: OrganizationType.fromString(json['type']),
      role: Role.fromString(json['role']),
      createdAt: DateTime.parse(json['createdAt']),
      updatedAt: DateTime.parse(json['updatedAt']),
    );
  }
}

class Project {
  final String id;
  final String name;
  final String orgId;
  final String slug;
  final DateTime createdAt;
  final DateTime updatedAt;

  Project._({
    required this.id,
    required this.name,
    required this.orgId,
    required this.slug,
    required this.createdAt,
    required this.updatedAt,
  });

  factory Project.fromJson(Map<dynamic, dynamic> json) {
    return Project._(
      id: json['id'],
      name: json['name'],
      orgId: json['orgId'],
      slug: json['slug'],
      createdAt: DateTime.parse(json['createdAt']),
      updatedAt: DateTime.parse(json['updatedAt']),
    );
  }
}

enum Role {
  admin,
  basicMember;

  static Role fromString(String role) {
    switch (role) {
      case 'admin':
        return Role.admin;
      case 'basic_member':
        return Role.basicMember;
      default:
        throw Exception('Invalid role: $role');
    }
  }
}

enum OrganizationType {
  personal,
  organization;

  static OrganizationType fromString(String type) {
    switch (type) {
      case 'personal':
        return OrganizationType.personal;
      case 'organization':
        return OrganizationType.organization;
      default:
        throw Exception('Invalid organization type: $type');
    }
  }
}
