import 'dart:convert';
import 'dart:io';

import 'package:cli_util/cli_util.dart';
import 'package:path/path.dart' as p;

/// A utility class for managing the user's authentication session.
class GlobeAuth {
  GlobeAuth() {
    _loadSession();
  }

  static const _applicationName = 'dart_globe';
  static const _sessionFileName = 'dart_globe_auth.json';

  DeploySession? _session;

  /// The current session, or `null` if the user is not logged in.
  DeploySession? get currentSession => _session;

  /// Logs the user in with the given [jwt].
  ///
  /// Note this is still validated on the server side.
  void login({required String jwt}) {
    _session = DeploySession(jwt: jwt);
    _flushSession(_session!);
  }

  /// Clears the local session.
  void logout() => _clearSession();

  /// The path to the local configuration directory.
  late final String? _localConfigDir = () {
    try {
      return applicationConfigHome(_applicationName);
    } catch (_) {
      return null;
    }
  }();

  /// Attempts to load the session from the local configuration directory.
  void _loadSession() {
    if (_localConfigDir == null) return;
    final sessionFile = File(p.join(_localConfigDir!, _sessionFileName));

    if (sessionFile.existsSync()) {
      try {
        final contents = sessionFile.readAsStringSync();
        _session = DeploySession.fromJson(
          json.decode(contents) as Map<String, dynamic>,
        );
      } catch (_) {}
    }
  }

  /// Flushes the given [session] to the local configuration directory.
  void _flushSession(DeploySession session) {
    File(p.join(_localConfigDir!, _sessionFileName))
      ..createSync(recursive: true)
      ..writeAsStringSync(json.encode(session.toJson()));
  }

  /// Clears the local session.
  void _clearSession() {
    _session = null;

    final sessionFile = File(p.join(_localConfigDir!, _sessionFileName));
    if (sessionFile.existsSync()) {
      sessionFile.deleteSync(recursive: true);
    }
  }
}

/// A data class representing a user's authentication session.
class DeploySession {
  /// Creates a session with the given [jwt].
  const DeploySession({required this.jwt});

  /// Creates a session from the given [json] object.
  factory DeploySession.fromJson(Map<String, dynamic> json) {
    return DeploySession(
      jwt: json['jwt'] as String,
    );
  }

  /// The users JWT token, used for authentication via the API.
  final String jwt;

  /// Converts this session to a JSON object.
  Map<String, dynamic> toJson() => {'jwt': jwt};
}
