import 'dart:io';
import 'dart:convert';

import 'package:path/path.dart' as p;

/// A utility class for managing the user's local project.
class Scope {
  /// Creates a new [Project] instance & loads in any current project metadata.
  Scope() {
    _loadScope();
  }

  static final Directory _projectDirectory = Directory(
    p.join('.dart_tool', 'dart_globe'),
  );

  static final File _projectFile = File(
    p.join(_projectDirectory.path, 'project.json'),
  );

  ScopeMetadata? _metadata;

  /// The current project metadata, or `null` if the project has not been setup.
  ScopeMetadata? get current => _metadata;

  /// Sets the current project scope.
  void setScope({required String orgId, required String projectId}) {
    _metadata = ScopeMetadata(orgId: orgId, projectId: projectId);
    _projectFile.createSync(recursive: true);
    _projectFile.writeAsStringSync(json.encode(_metadata!.toJson()));
  }

  /// Clears the current project metadata.
  void clear() {
    _metadata = null;
    _projectFile.deleteSync(recursive: true);
  }

  /// Sets the current scope metadata.
  void _loadScope() {
    if (_projectFile.existsSync()) {
      try {
        final contents = _projectFile.readAsStringSync();
        _metadata = ScopeMetadata.fromJson(
          json.decode(contents) as Map<String, dynamic>,
        );
      } catch (_) {}
    }
  }
}

/// A data class containing the current project metadata.
class ScopeMetadata {
  /// Creates a new [ScopeMetadata] instance.
  const ScopeMetadata({
    required this.orgId,
    required this.projectId,
  });

  /// Creates a new [ScopeMetadata] instance from the given [json].
  factory ScopeMetadata.fromJson(Map<String, dynamic> json) {
    return ScopeMetadata(
      orgId: json['orgId'] as String,
      projectId: json['projectId'] as String,
    );
  }

  /// The organization ID this project belongs to.
  final String orgId;

  /// The project ID, which belongs to the org.
  final String projectId;

  /// Converts this [ScopeMetadata] instance to a JSON map.
  Map<String, dynamic> toJson() => {'orgId': orgId, 'projectId': projectId};
}
